//
// hello.mag.45.c   -- accel.c
//
// Hall effect magnetic sensing hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld 11/3/13
// (c) Massachusetts Institute of Technology 2013
//
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all 
// liability.
//


//Modified by Victoria Peredo
//FabLab Madrid CEU

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 102 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

//I changed the port to the one that I am using for RX: PORTA, pin PA7

#define serial_port PORTA
#define serial_direction DDRA
#define serial_pin_out (1 << PA7)


void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

   //Function to read the ADC signal, depending on the channel that is selected we read ADC3, 3 or 1

   uint16_t readadc(uint8_t ch)
   {
      if(ch == 3){
        ADMUX = 0b01000011; // I wrote the registers value directly
         //ADMUX |= (0 << MUX3) | (0 << MUX2) | (1 << MUX1) | (1 << MUX0); // ADC3 - x (011)
      }
      else if(ch == 2){
        ADMUX = 0b01000010;
         //ADMUX |= (0 << MUX3) | (0 << MUX2) | (1 << MUX1) | (0 << MUX0); // ADC2 - y (010)
      }
      else{
        ADMUX = 0b01000001;
        //ADMUX |= (0 << MUX3) | (0 << MUX2) | (0 << MUX1) | (1 << MUX0); // ADC1 - z (001)
      }

      ADCSRA|=(1<<ADSC);            //START CONVERSION
      while((ADCSRA)&(1<<ADSC))
         ; 
      
         //WAIT UNTIL CONVERSION IS COMPLETE
      _delay_ms(1);
      return(ADC);                  //Return the whole ADC value (16 bits)
   }

int main(void) {
   
   //Defining the variables where I will store the ADC signal
   uint16_t x,y,z;
   //
   // main
   //
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize output pins, calling the previously defined functions with the serial output (porta, pa7)
   //
   set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);
   //
   // We initialize the ADC, we just do this the first time
   //
   ADMUX = (0 << REFS1) | (1 << REFS0); // I am using an external reference por the voltage in the PA0
   ADCSRB =  (0 << ADLAR) ;// right adjust
   ADCSRA = (1 << ADEN) // enable ADC
      | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128
   //
   // main loop
   //
   while (1) {
      
    //for 100  hilo = 32 bits
      x=readadc(3);      //when we send a 3 to the function, the ADC3 that is connected to the x signal will be read
      y=readadc(2);      //with a 2 the ADC2, the y signal
      z=readadc(1);      //and with a 1 the ADC1, the 1 signal


      //
      // send framing
      //
      put_char(&serial_port, serial_pin_out, 1);
      char_delay();
      put_char(&serial_port, serial_pin_out, 2);
      char_delay();
      put_char(&serial_port, serial_pin_out, 3);
      char_delay();
      put_char(&serial_port, serial_pin_out, 4);
      char_delay();
      
      //
      // send result, as we are sending the 16 bits for each of the signals (x,y,z) we need to divide them in blocks of
      // two to be able to send them



      put_char(&serial_port, serial_pin_out, (x & 255));
      char_delay();
      put_char(&serial_port, serial_pin_out, ((x >> 8) & 255));
      char_delay();
      put_char(&serial_port, serial_pin_out, (y & 255));
      char_delay();
      put_char(&serial_port, serial_pin_out, ((y >> 8) & 255)); 
      char_delay();  
      put_char(&serial_port, serial_pin_out, (z & 255)); 
      char_delay();
      put_char(&serial_port, serial_pin_out, ((z >> 8) & 255));
      char_delay();   

   }
}

